from enum import IntEnum


class CommandBeginColumns(IntEnum):
    EVENTCLASS = 0
    EVENTSUBCLASS = 1
    CURRENTTIME = 2
    STARTTIME = 3
    SESSITON = 8
    CONNECTIONID = 25
    DATABASENAME = 28
    NTUSERNAME = 32
    NTDOMAINNAME = 33
    CLIENTPROCESSID = 36
    APPLICATIONNAME = 37
    SESSIONID = 39
    NTCANONICALUSERNAME = 40
    SPID = 41
    TEXTDATA = 42
    SERVERNAME = 43
    REQUESTPARAMETERS = 44
    REQUESTPROPERTIES = 45
    ACTIVITYID = 46
    REQUESTID = 47
    USEROBJECTID = 51
    APPLICATIONCONTEXT = 52
    DATABASEFRIENDLYNAME = 54
    IDENTITY = 55


class CommandEndColumns(IntEnum):
    EVENTCLASS = 0
    EVENTSUBCLASS = 1
    CURRENTTIME = 2
    STARTTIME = 3
    ENDTIME = 4
    DURATION = 5
    CPUTIME = 6
    SESSITON = 8
    INTEGERDATA = 10
    SEVERITY = 22
    SUCCESS = 23
    ERROR = 24
    CONNECTIONID = 25
    DATABASENAME = 28
    NTUSERNAME = 32
    NTDOMAINNAME = 33
    CLIENTPROCESSID = 36
    APPLICATIONNAME = 37
    SESSIONID = 39
    NTCANONICALUSERNAME = 40
    SPID = 41
    TEXTDATA = 42
    SERVERNAME = 43
    ACTIVITYID = 46
    REQUESTID = 47
    ERRORTYPE = 49
    USEROBJECTID = 51
    APPLICATIONCONTEXT = 52
    DATABASEFRIENDLYNAME = 54
    IDENTITY = 55
