from enum import IntEnum


class VertipaqSeQueryBeginColumns(IntEnum):
    EVENTCLASS = 0
    EVENTSUBCLASS = 1
    CURRENTTIME = 2
    STARTTIME = 3
    JOBID = 7
    SESSIONTYPE = 8
    OBJECTID = 11
    OBJECTTYPE = 12
    OBJECTNAME = 13
    OBJECTPATH = 14
    OBJECTREFERENCE = 15
    CONNECTIONID = 25
    DATABASENAME = 28
    NTUSERNAME = 32
    NTDOMAINNAME = 33
    SESSIONID = 39
    NTCANONICALUSERNAME = 40
    SPID = 41
    TEXTDATA = 42
    SERVERNAME = 43
    ACTIVITYID = 46
    REQUESTID = 47
    USEROBJECTID = 51
    APPLICATIONCONTEXT = 52
    DATABASEFRIENDLYNAME = 54
    IDENTITY = 55


class VertipaqSeQueryEndColumns(IntEnum):
    EVENTCLASS = 0
    EVENTSUBCLASS = 1
    CURRENTTIME = 2
    STARTTIME = 3
    ENDTIME = 4
    DURATION = 5
    CPUTIME = 6
    JOBID = 7
    SESSIONTYPE = 8
    PROGRESSTOTAL = 9
    INTEGERDATA = 10
    OBJECTID = 11
    OBJECTTYPE = 12
    OBJECTNAME = 13
    OBJECTPATH = 14
    OBJECTREFERENCE = 15
    SEVERITY = 22
    SUCCESS = 23
    ERROR = 24
    CONNECTIONID = 25
    DATABASENAME = 28
    NTUSERNAME = 32
    NTDOMAINNAME = 33
    SESSIONID = 39
    NTCANONICALUSERNAME = 40
    SPID = 41
    TEXTDATA = 42
    SERVERNAME = 43
    ACTIVITYID = 46
    REQUESTID = 47
    ERRORTYPE = 49
    USEROBJECTID = 51
    APPLICATIONCONTEXT = 52
    DATABASEFRIENDLYNAME = 54
    IDENTITY = 55


class VertipaqSeQueryCacheMissColumns(IntEnum):
    EVENTCLASS = 0
    EVENTSUBCLASS = 1
    CURRENTTIME = 2
    JOBID = 7
    SESSIONTYPE = 8
    OBJECTID = 11
    OBJECTTYPE = 12
    OBJECTNAME = 13
    OBJECTPATH = 14
    OBJECTREFERENCE = 15
    CONNECTIONID = 25
    DATABASENAME = 28
    NTUSERNAME = 32
    NTDOMAINNAME = 33
    SESSIONID = 39
    NTCANONICALUSERNAME = 40
    SPID = 41
    TEXTDATA = 42
    SERVERDATA = 43
    ACTIVITYID = 46
    REQUESTID = 47
    USEROBJECTID = 51
    APPLICATIONCONTEXT = 52
    DATABASEFRIENDLYNAME = 54
    IDENTITY = 55


class VertipaqSeQueryCacheMatchColumns(IntEnum):
    EVENTCLASS = 0
    EVENTSUBCLASS = 1
    CURRENTTIME = 2
    JOBID = 7
    SESSIONTYPE = 8
    OBJECTID = 11
    OBJECTTYPE = 12
    OBJECTNAME = 13
    OBJECTPATH = 14
    OBJECTREFERENCE = 15
    CONNECTIONID = 25
    DATABASENAME = 28
    NTUSERNAME = 32
    NTDOMAINNAME = 33
    SESSIONID = 39
    NTCANONICALUSERNAME = 40
    SPID = 41
    TEXTDATA = 42
    SERVERDATA = 43
    ACTIVITYID = 46
    REQUESTID = 47
    USEROBJECTID = 51
    APPLICATIONCONTEXT = 52
    DATABASEFRIENDLYNAME = 54
    IDENTITY = 55


class CalculationEvaluationColumns(IntEnum):
    EVENTCLASS = 0
    EVENTSUBCLASS = 1
    CURRENTTIME = 2
    STARTTIME = 3
    ENDTIME = 4
    DURATION = 5
    CPUTIME = 6
    INTEGERDATA = 10
    CONNECTIONID = 25
    DATABASENAME = 28
    CLIENTHOSTNAME = 35
    CLIENTPROCESSID = 36
    APPLICATIONNAME = 37
    SESSIONID = 39
    NTCANONICALUSERNAME = 40
    SPID = 41
    TEXTDATA = 42
    SERVERDATA = 43
    ACTIVITYID = 46
    REQUESTID = 47
    APPLICATIONCONTEXT = 52


class CalculationEvaluationDetailedInformationColumns(IntEnum):
    EVENTCLASS = 0
    EVENTSUBCLASS = 1
    CURRENTTIME = 2
    STARTTIME = 3
    ENDTIME = 4
    DURATION = 5
    CPUTIME = 6
    INTEGERDATA = 10
    CONNECTIONID = 25
    DATABASENAME = 28
    CLIENTHOSTNAME = 35
    CLIENTPROCESSID = 36
    APPLICATIONNAME = 37
    SESSIONID = 39
    NTCANONICALUSERNAME = 40
    SPID = 41
    TEXTDATA = 42
    SERVERDATA = 43
    ACTIVITYID = 46
    REQUESTID = 47
    APPLICATIONCONTEXT = 52


class DaxQueryPlanColumns(IntEnum):
    EVENTCLASS = 0
    EVENTSUBCLASS = 1
    CURRENTTIME = 2
    STARTTIME = 3
    ENDTIME = 4
    DURATION = 5
    CPUTIME = 6
    INTEGERDATA = 10
    CONNECTIONID = 25
    DATABASENAME = 28
    CLIENTHOSTNAME = 35
    CLIENTPROCESSID = 36
    APPLICATIONNAME = 37
    SESSIONID = 39
    NTCANONICALUSERNAME = 40
    SPID = 41
    TEXTDATA = 42
    SERVERDATA = 43
    ACTIVITYID = 46
    REQUESTID = 47
    APPLICATIONCONTEXT = 52


class DaxExtensionExecutionBeginColumns(IntEnum):
    EVENTCLASS = 0
    CURRENTTIME = 2
    INTEGERDATA = 10
    DATABASENAME = 28
    TEXTDATA = 42
    SERVERDATA = 43
    ACTIVITYID = 46
    REQUESTID = 47
    FUNCTIONNAME = 50
    APPLICATIONCONTEXT = 52


class DaxExtensionExecutionEndColumns(IntEnum):
    EVENTCLASS = 0
    CURRENTTIME = 2
    STARTTIME = 3
    ENDTIME = 4
    DURATION = 5
    CPUTIME = 6
    INTEGERDATA = 10
    SEVERITY = 22
    SUCCESS = 23
    ERROR = 24
    DATABASENAME = 28
    TEXTDATA = 42
    SERVERDATA = 43
    ACTIVITYID = 46
    REQUESTID = 47
    ERRORTYPE = 49
    FUNCTIONNAME = 50
    APPLICATIONCONTEXT = 52


class DaxExtensionTraceErrorColumns(IntEnum):
    EVENTCLASS = 0
    CURRENTTIME = 2
    INTEGERDATA = 10
    DATABASENAME = 28
    TEXTDATA = 42
    SERVERDATA = 43
    ACTIVITYID = 46
    REQUESTID = 47
    FUNCTIONNAME = 50
    APPLICATIONCONTEXT = 52


class DaxExtensionTraceInfoColumns(IntEnum):
    EVENTCLASS = 0
    CURRENTTIME = 2
    INTEGERDATA = 10
    DATABASENAME = 28
    TEXTDATA = 42
    SERVERDATA = 43
    ACTIVITYID = 46
    REQUESTID = 47
    FUNCTIONNAME = 50
    APPLICATIONCONTEXT = 52


class DaxExtensionTraceVerboseColumns(IntEnum):
    EVENTCLASS = 0
    CURRENTTIME = 2
    INTEGERDATA = 10
    DATABASENAME = 28
    TEXTDATA = 42
    SERVERDATA = 43
    ACTIVITYID = 46
    REQUESTID = 47
    FUNCTIONNAME = 50
    APPLICATIONCONTEXT = 52


class AggregateTableRewriteQueryColumns(IntEnum):
    EVENTCLASS = 0
    EVENTSUBCLASS = 1
    CURRENTTIME = 2
    STARTTIME = 3
    ENDTIME = 4
    DURATION = 5
    INTEGERDATA = 10
    SUCCESS = 23
    CONNECTIONID = 25
    DATABASENAME = 28
    NTUSERNAME = 32
    NTDOMAINNAME = 33
    CLIENTHOSTNAME = 35
    CLIENTPROCESSID = 36
    APPLICATIONNAME = 37
    SESSIONID = 39
    NTCANONICALUSERNAME = 40
    SPID = 41
    TEXTDATA = 42
    SERVERDATA = 43
    ACTIVITYID = 46
    REQUESTID = 47
    USEROBJECTID = 51
    APPLICATIONCONTEXT = 52


class AggregateTableRewriteInfoColumns(IntEnum):
    EVENTCLASS = 0
    CURRENTTIME = 2
    STARTTIME = 3
    SEVERITY = 22
    CONNECTIONID = 25
    DATABASENAME = 28
    NTUSERNAME = 32
    NTDOMAINNAME = 33
    CLIENTHOSTNAME = 35
    CLIENTPROCESSID = 36
    APPLICATIONNAME = 37
    SESSIONID = 39
    NTCANONICALUSERNAME = 40
    SPID = 41
    TEXTDATA = 42
    SERVERDATA = 43
    ACTIVITYID = 46
    REQUESTID = 47
    APPLICATIONCONTEXT = 52


class QuerySubcubeColumns(IntEnum):
    EVENTCLASS = 0
    EVENTSUBCLASS = 1
    CURRENTTIME = 2
    STARTTIME = 3
    ENDTIME = 4
    DURATION = 5
    CPUTIME = 6
    OBJECTPATH = 14
    CONNECTIONID = 25
    DATABASENAME = 28
    SESSIONID = 39
    NTCANONICALUSERNAME = 40
    SPID = 41
    TEXTDATA = 42
    SERVERDATA = 43
    ACTIVITYID = 46
    REQUESTID = 47
    APPLICATIONCONTEXT = 52
    DATABASEFRIENDLYNAME = 54
    IDENTITY = 55


class DirectQueryBeginColumns(IntEnum):
    EVENTCLASS = 0
    CURRENTIME = 2
    STARTTIME = 3
    ENDTIME = 4
    DURATION = 5
    CPUTIME = 6
    JOBID = 7
    SESSIONTYPE = 8
    INTEGERDATA = 10
    OBJECTID = 11
    OBJECTTYPE = 12
    OBJECTNAME = 13
    OBJECTPATH = 14
    SEVERITY = 22
    SUCCESS = 23
    ERROR = 24
    CONNECTIONID = 25
    DATABASENAME = 28
    NTUSERNAME = 32
    CLIENTPROCESSID = 36
    SESSIONID = 39
    SPID = 41
    TEXTDATA = 42
    SERVERNAME = 43
    ACTIVITYID = 46
    REQUESTID = 47
    ERRORTYPE = 49
    APPLICATIONCONTEXT = 52
    DATABASEFRIENDLYNAME = 54
    IDENTITY = 55


class DirectQueryEndColumns(IntEnum):
    EVENTCLASS = 0
    CURRENTIME = 2
    STARTTIME = 3
    ENDTIME = 4
    DURATION = 5
    CPUTIME = 6
    JOBID = 7
    SESSIONTYPE = 8
    INTEGERDATA = 10
    OBJECTID = 11
    OBJECTTYPE = 12
    OBJECTNAME = 13
    OBJECTPATH = 14
    SEVERITY = 22
    SUCCESS = 23
    ERROR = 24
    CONNECTIONID = 25
    DATABASENAME = 28
    NTUSERNAME = 32
    CLIENTPROCESSID = 36
    SESSIONID = 39
    SPID = 41
    TEXTDATA = 42
    SERVERNAME = 43
    ACTIVITYID = 46
    REQUESTID = 47
    ERRORTYPE = 49
    APPLICATIONCONTEXT = 52
    DATABASEFRIENDLYNAME = 54
    IDENTITY = 55


class ExecutionMetricsColumns(IntEnum):
    EVENTCLASS = 0
    DATABASENAME = 28
    APPLICATIONNAME = 37
    SPID = 41
    TEXTDATA = 42
    ACTIVITYID = 46
    REQUESTID = 47
